@@ -16,9 +16,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "../config.h"
+#include <config.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
+#include <sys/socket.h>
 #ifdef HAVE_SYS_TERMIOS_H
 #include <sys/termios.h>
 #endif
@@ -49,15 +50,9 @@
 #define PTY_RECVMSG_FLAGS 0
 #endif
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(String) dgettext(PACKAGE, String)
-#else
-#define _(String) String
-#endif
+#include <glib/gi18n-lib.h>
 
 #ifdef VTE_USE_GNOME_PTY_HELPER
-#include <sys/socket.h>
 #include <sys/uio.h>
 #ifdef HAVE_SYS_UN_H
 #include <sys/un.h>
@@ -69,8 +64,6 @@ static int _vte_pty_helper_tunnel = -1;
 static GTree *_vte_pty_helper_map = NULL;
 #endif
 
-extern char **environ;
-
 /* Reset the handlers for all known signals to their defaults.  The parent
  * (or one of the libraries it links to) may have changed one to be ignored. */
 static void
@@ -276,19 +269,20 @@ static gchar **
 merge_environ (char **envp)
 {
 	GHashTable *table;
+	gchar **parent_environ;
 	GPtrArray *array;
 	gint i;
 
 	table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-	for (i = 0; environ[i] != NULL; i++) {
-		gchar *name = g_strdup (environ[i]);
-		gchar *value = strchr (name, '=');
-		if (value) {
-			*value = '\0';
-			value = g_strdup (value + 1);
-		}
-		g_hash_table_replace (table, name, value);
+
+	parent_environ = g_listenv ();
+	for (i = 0; parent_environ[i] != NULL; i++) {
+		g_hash_table_replace (table,
+			              g_strdup (parent_environ[i]),
+				      g_strdup (g_getenv (parent_environ[i])));
 	}
+	g_strfreev (parent_environ);
+
 	if (envp != NULL) {
 		for (i = 0; envp[i] != NULL; i++) {
 			gchar *name = g_strdup (envp[i]);
@@ -642,6 +636,7 @@ _vte_pty_open_unix98(GPid *child, char **env_add,
 	return fd;
 }
 
+#ifdef VTE_USE_GNOME_PTY_HELPER
 #ifdef HAVE_RECVMSG
 static void
 _vte_pty_read_ptypair(int tunnel, int *parentfd, int *childfd)
@@ -701,7 +696,6 @@ _vte_pty_read_ptypair(int tunnel, int *parentfd, int *childfd)
 }
 #endif
 
-#ifdef VTE_USE_GNOME_PTY_HELPER
 #ifdef HAVE_SOCKETPAIR
 static int
 _vte_pty_pipe_open(int *a, int *b)
@@ -858,11 +852,10 @@ _vte_pty_start_helper(void)
 		return FALSE;
 	}
 	if (_vte_pty_helper_pid == 0) {
-		/* Child.  Close all descriptors. */
-		for (i = 0; i < sysconf(_SC_OPEN_MAX); i++) {
-			if (i != tunnel) {
-				close(i);
-			}
+		/* Child.  Close descriptors.  No need to close all,
+		 * gnome-pty-helper does that anyway. */
+		for (i = 0; i < 3; i++) {
+			close(i);
 		}
 		/* Reassign the socket pair to stdio. */
 		dup2(tunnel, STDIN_FILENO);
@@ -1005,7 +998,7 @@ _vte_pty_open(pid_t *child_pid, char **env_add,
 	}
 	g_assert(op >= 0);
 	g_assert(op < G_N_ELEMENTS(opmap));
-	if (ret == -1) {
+	if (ret == -1 && op != 0) {
 		ret = _vte_pty_open_with_helper(&child, env_add, command, argv,
 						directory,
 						columns, rows, opmap[op]);
@@ -1071,9 +1064,10 @@ _vte_pty_close(int pty)
 		if (g_tree_lookup(_vte_pty_helper_map, GINT_TO_POINTER(pty))) {
 			/* Signal the helper that it needs to close its
 			 * connection. */
-			ops = GNOME_PTY_CLOSE_PTY;
 			tag = g_tree_lookup(_vte_pty_helper_map,
 					    GINT_TO_POINTER(pty));
+
+			ops = GNOME_PTY_CLOSE_PTY;
 			if (n_write(_vte_pty_helper_tunnel,
 				    &ops, sizeof(ops)) != sizeof(ops)) {
 				return;
@@ -1082,6 +1076,14 @@ _vte_pty_close(int pty)
 				    &tag, sizeof(tag)) != sizeof(tag)) {
 				return;
 			}
+
+			ops = GNOME_PTY_SYNCH;
+			if (n_write(_vte_pty_helper_tunnel,
+				    &ops, sizeof(ops)) != sizeof(ops)) {
+				return;
+			}
+			n_read(_vte_pty_helper_tunnel, &ops, 1);
+
 			/* Remove the item from the map. */
 			g_tree_remove(_vte_pty_helper_map,
 				      GINT_TO_POINTER(pty));
@@ -1107,7 +1109,7 @@ main(int argc, char **argv)
 	char c;
 	int ret;
 	signal(SIGCHLD, sigchld_handler);
-	_vte_debug_parse_string(getenv("VTE_DEBUG_FLAGS"));
+	_vte_debug_init();
 	fd = _vte_pty_open(&child, NULL,
 			   (argc > 1) ? argv[1] : NULL,
 			   (argc > 1) ? argv + 1 : NULL,
