@@ -42,10 +42,10 @@ struct _VteBgClass {
 };
 
 #define VTE_TYPE_BG vte_bg_get_type()
-#define VTE_BG(obj)	       (GTK_CHECK_CAST((obj), VTE_TYPE_BG, VteBg))
-#define VTE_BG_CLASS(klass)    (GTK_CHECK_CLASS_CAST((klass), VTE_TYPE_BG, VteBgClass)
-#define VTE_IS_BG(obj)	       GTK_CHECK_TYPE((obj), VTE_TYPE_BG)
-#define VTE_IS_BG_CLASS(klass) GTK_CHECK_CLASS_TYPE((klass), VTE_TYPE_BG)
+#define VTE_BG(obj)	       (G_TYPE_CHECK_INSTANCE_CAST((obj), VTE_TYPE_BG, VteBg))
+#define VTE_BG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), VTE_TYPE_BG, VteBgClass)
+#define VTE_IS_BG(obj)	       G_TYPE_CHECK_INSTANCE_TYPE((obj), VTE_TYPE_BG)
+#define VTE_IS_BG_CLASS(klass) G_TYPE_CHECK_CLASS_TYPE((klass), VTE_TYPE_BG)
 #define VTE_BG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), VTE_TYPE_BG, VteBgClass))
 
 GType vte_bg_get_type(void);
@@ -65,11 +65,6 @@ GdkPixmap *vte_bg_get_pixmap(VteBg *bg, enum VteBgSourceType source_type,
 			     const GdkColor *tint, double saturation,
 			     GdkColormap *colormap);
 
-GdkPixbuf *vte_bg_get_pixbuf(VteBg *bg, enum VteBgSourceType source_type,
-			     GdkPixbuf *source_pixbuf,
-			     const char *source_file,
-			     const GdkColor *tint, double saturation);
-
 G_END_DECLS
 
 #endif
