@@ -19,7 +19,7 @@
 /* The interfaces in this file are subject to change at any time. */
 
 
-#include "../config.h"
+#include <config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,32 +28,12 @@
 #include <gtk/gtk.h>
 #include "debug.h"
 #include "vtedraw.h"
-#include "vteft2.h"
-#include "vtegl.h"
-#include "vtepango.h"
-#include "vtepangox.h"
+#include "vtepangocairo.h"
 #include "vteskel.h"
-#include "vtexft.h"
 
 static const struct _vte_draw_impl
 *_vte_draw_impls[] = {
-#ifndef X_DISPLAY_MISSING
-#ifdef HAVE_XFT2
-	&_vte_draw_xft,
-#endif /* HAVE_XFT2 */
-#endif /* !X_DISPLAY_MISSING */
-	&_vte_draw_ft2,
-#ifndef X_DISPLAY_MISSING
-#ifdef HAVE_GL
-	&_vte_draw_gl,
-#endif /* HAVE_GL */
-#endif /* !X_DISPLAY_MISSING */
-	&_vte_draw_pango,
-#ifndef X_DISPLAY_MISSING
-#ifdef HAVE_PANGOX
-	&_vte_draw_pango_x,
-#endif /* HAVE_PANGOX */
-#endif /* !X_DISPLAY_MISSING */
+	&_vte_draw_pangocairo,
 };
 
 static gboolean
@@ -94,7 +74,8 @@ _vte_draw_init_user (struct _vte_draw *draw)
 		/* find among available draws */
 		for (i = 0; i < G_N_ELEMENTS (_vte_draw_impls); i++) {
 			if (strcmp (*s, _vte_draw_impls[i]->name) == 0) {
-				if (_vte_draw_impls[i]->check (draw, draw->widget)) {
+				if (_vte_draw_impls[i]->check == NULL ||
+				    _vte_draw_impls[i]->check (draw, draw->widget)) {
 					draw->impl = _vte_draw_impls[i];
 					goto out;
 				}
@@ -115,7 +96,8 @@ _vte_draw_init_default (struct _vte_draw *draw)
 	guint i;
 
 	for (i = 0; i < G_N_ELEMENTS (_vte_draw_impls); i++) {
-		if (_vte_draw_impls[i]->check (draw, draw->widget)) {
+		if (_vte_draw_impls[i]->check == NULL ||
+		    _vte_draw_impls[i]->check (draw, draw->widget)) {
 			draw->impl = _vte_draw_impls[i];
 			return TRUE;
 		}
@@ -133,7 +115,6 @@ _vte_draw_new (GtkWidget *widget)
 	/* Create the structure. */
 	draw = g_slice_new0 (struct _vte_draw);
 	draw->widget = g_object_ref (widget);
-	draw->started = FALSE;
 
 	/* Allow the user to specify her preferred backends */
 	if (!_vte_draw_init_user (draw) &&
@@ -144,11 +125,14 @@ _vte_draw_new (GtkWidget *widget)
 		draw->impl = &_vte_draw_skel;
 	}
 
+	draw->requires_clear = draw->impl->always_requires_clear;
+
 	_vte_debug_print (VTE_DEBUG_DRAW,
 			"draw_new (%s)\n", draw->impl->name);
 	_vte_debug_print (VTE_DEBUG_MISC, "Using %s.\n", draw->impl->name);
 
-	draw->impl->create (draw, draw->widget);
+	if (draw->impl->create)
+		draw->impl->create (draw, draw->widget);
 
 	return draw;
 }
@@ -157,7 +141,9 @@ void
 _vte_draw_free (struct _vte_draw *draw)
 {
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_free\n");
-	draw->impl->destroy (draw);
+
+	if (draw->impl->destroy)
+		draw->impl->destroy (draw);
 
 	if (draw->widget != NULL) {
 		g_object_unref (draw->widget);
@@ -169,29 +155,32 @@ _vte_draw_free (struct _vte_draw *draw)
 GdkVisual *
 _vte_draw_get_visual (struct _vte_draw *draw)
 {
-	g_return_val_if_fail (draw->impl != NULL, NULL);
-	g_return_val_if_fail (draw->impl->get_visual != NULL, NULL);
+	GdkVisual *visual = NULL;
+
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_get_visual\n");
-	return draw->impl->get_visual (draw);
+
+	if (draw->impl->get_visual)
+		visual = draw->impl->get_visual (draw);
+
+	return visual ? visual : gtk_widget_get_visual (draw->widget);
 }
 
 GdkColormap *
 _vte_draw_get_colormap (struct _vte_draw *draw, gboolean maybe_use_default)
 {
 	GdkColormap *colormap;
-	GdkScreen *screen;
-	g_return_val_if_fail (draw->impl != NULL, NULL);
-	g_return_val_if_fail (draw->impl->get_colormap != NULL, NULL);
+
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_get_colormap\n");
-	colormap = draw->impl->get_colormap (draw);
-	if (colormap) {
-		return colormap;
+
+	if (draw->impl->get_colormap)
+		colormap = draw->impl->get_colormap (draw);
+	else
+		colormap = gtk_widget_get_colormap (draw->widget);
+
+	if (colormap == NULL && maybe_use_default) {
+		colormap = gdk_screen_get_default_colormap (gtk_widget_get_screen (draw->widget));
 	}
-	if (!maybe_use_default) {
-		return NULL;
-	}
-	screen = gtk_widget_get_screen (draw->widget);
-	colormap = gdk_screen_get_default_colormap (screen);
+
 	return colormap;
 }
 
@@ -199,11 +188,14 @@ void
 _vte_draw_start (struct _vte_draw *draw)
 {
 	g_return_if_fail (GTK_WIDGET_REALIZED (draw->widget));
-	g_return_if_fail (draw->impl != NULL);
-	g_return_if_fail (draw->impl->start != NULL);
+
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_start\n");
+
 	g_object_ref (draw->widget->window);
-	draw->impl->start (draw);
+
+	if (draw->impl->start)
+		draw->impl->start (draw);
+
 	draw->started = TRUE;
 }
 
@@ -211,36 +203,54 @@ void
 _vte_draw_end (struct _vte_draw *draw)
 {
 	g_return_if_fail (draw->started == TRUE);
-	g_return_if_fail (draw->impl != NULL);
-	g_return_if_fail (draw->impl->end != NULL);
-	draw->impl->end (draw);
+
+	if (draw->impl->end)
+		draw->impl->end (draw);
+
 	g_object_unref (draw->widget->window);
+
 	draw->started = FALSE;
+
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_end\n");
 }
 
 void
-_vte_draw_set_background_color (struct _vte_draw *draw,
+_vte_draw_set_background_solid(struct _vte_draw *draw,
 			       GdkColor *color,
 			       guint16 opacity)
 {
-	g_return_if_fail (draw->impl != NULL);
-	g_return_if_fail (draw->impl->set_background_color != NULL);
-	draw->impl->set_background_color (draw, color, opacity);
+	draw->requires_clear = draw->impl->always_requires_clear || opacity != 0xFFFF;
+
+	if (draw->impl->set_background_solid)
+		draw->impl->set_background_solid (draw, color, opacity);
 }
 
 void
 _vte_draw_set_background_image (struct _vte_draw *draw,
-			       enum VteBgSourceType type,
-			       GdkPixbuf *pixbuf,
-			       const char *filename,
-			       const GdkColor *color,
-			       double saturation)
+			        enum VteBgSourceType type,
+			        GdkPixbuf *pixbuf,
+			        const char *filename,
+			        const GdkColor *color,
+			        double saturation)
 {
-	g_return_if_fail (draw->impl != NULL);
-	g_return_if_fail (draw->impl->set_background_image != NULL);
-	draw->impl->set_background_image (draw, type, pixbuf, filename,
-					 color, saturation);
+	if (type != VTE_BG_SOURCE_NONE)
+		draw->requires_clear = TRUE;
+
+	if (draw->impl->set_background_image)
+		draw->impl->set_background_image (draw, type, pixbuf, filename,
+						  color, saturation);
+}
+
+void
+_vte_draw_set_background_scroll (struct _vte_draw *draw,
+				 gint x, gint y)
+{
+	_vte_debug_print (VTE_DEBUG_DRAW,
+			"draw_set_scroll (%d, %d)\n",
+			x, y);
+
+	if (draw->impl->set_background_scroll)
+		draw->impl->set_background_scroll (draw, x, y);
 }
 
 gboolean
@@ -249,34 +259,28 @@ _vte_draw_requires_clear (struct _vte_draw *draw)
 	return draw->requires_clear;
 }
 
-gboolean
-_vte_draw_requires_repaint (struct _vte_draw *draw)
-{
-	g_return_val_if_fail (draw->impl != NULL, TRUE);
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_requires_repaint = %d\n",
-			draw->impl->requires_repaint);
-	return draw->impl->requires_repaint;
-}
-
 gboolean
 _vte_draw_clip (struct _vte_draw *draw, GdkRegion *region)
 {
-	g_return_val_if_fail (draw->impl != NULL, FALSE);
+	gboolean clip = FALSE;
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_clip\n");
-	if (draw->impl->clip == NULL) {
-		return FALSE;
+
+	if (draw->impl->clip) {
+		draw->impl->clip (draw, region);
+		clip = TRUE;
 	}
-	draw->impl->clip (draw, region);
-	return TRUE;
+
+	return clip;
 }
 
 void
 _vte_draw_clear (struct _vte_draw *draw, gint x, gint y, gint width, gint height)
 {
-	g_return_if_fail (draw->impl != NULL);
 	g_return_if_fail (draw->impl->clear != NULL);
+
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_clear (%d, %d, %d, %d)\n",
-			x,y,width, height);
+			  x,y,width, height);
+
 	draw->impl->clear (draw, x, y, width, height);
 }
 
@@ -285,51 +289,40 @@ _vte_draw_set_text_font (struct _vte_draw *draw,
 			const PangoFontDescription *fontdesc,
 			VteTerminalAntiAlias anti_alias)
 {
-	g_return_if_fail (draw->impl != NULL);
-	g_return_if_fail (draw->impl->set_text_font != NULL);
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_set_text_font (aa=%d)\n",
-			anti_alias);
-	draw->impl->set_text_font (draw, fontdesc, anti_alias);
+			  anti_alias);
+
+	if (draw->impl->set_text_font)
+		draw->impl->set_text_font (draw, fontdesc, anti_alias);
+}
+
+void
+_vte_draw_get_text_metrics(struct _vte_draw *draw,
+			   gint *width, gint *height, gint *ascent)
+{
+	gint swidth = 0, sheight = 0, sascent = 0;
+
+	g_return_if_fail (draw->impl->get_text_metrics != NULL);
+
+	draw->impl->get_text_metrics (draw, &swidth, &sheight, &sascent);
+
+	if (width)  *width  = swidth;
+	if (height) *height = sheight;
+	if (ascent) *ascent = sascent;
 }
 
 int
-_vte_draw_get_text_width (struct _vte_draw *draw)
+_vte_draw_get_char_width (struct _vte_draw *draw, vteunistr c, int columns)
 {
-	g_return_val_if_fail (draw->impl != NULL, 1);
-	g_return_val_if_fail (draw->impl->get_text_width != NULL, 1);
-	return draw->impl->get_text_width (draw);
-}
+	int width = 0;
 
-int
-_vte_draw_get_text_height (struct _vte_draw *draw)
-{
-	g_return_val_if_fail (draw->impl != NULL, 1);
-	g_return_val_if_fail (draw->impl->get_text_height != NULL, 1);
-	return draw->impl->get_text_height (draw);
-}
+	if (draw->impl->get_char_width)
+		width = draw->impl->get_char_width (draw, c, columns);
 
-int
-_vte_draw_get_text_ascent (struct _vte_draw *draw)
-{
-	g_return_val_if_fail (draw->impl != NULL, 1);
-	g_return_val_if_fail (draw->impl->get_text_ascent != NULL, 1);
-	return draw->impl->get_text_ascent (draw);
-}
+	if (width == 0)
+		_vte_draw_get_text_metrics (draw, &width, NULL, NULL);
 
-int
-_vte_draw_get_char_width (struct _vte_draw *draw, gunichar c, int columns)
-{
-	g_return_val_if_fail (draw->impl != NULL, 1);
-	g_return_val_if_fail (draw->impl->get_char_width != NULL, 1);
-	return draw->impl->get_char_width (draw, c, columns);
-}
-
-gboolean
-_vte_draw_get_using_fontconfig (struct _vte_draw *draw)
-{
-	g_return_val_if_fail (draw->impl != NULL, TRUE);
-	g_return_val_if_fail (draw->impl->get_using_fontconfig != NULL, FALSE);
-	return draw->impl->get_using_fontconfig (draw);
+	return width;
 }
 
 void
@@ -338,8 +331,8 @@ _vte_draw_text (struct _vte_draw *draw,
 	       GdkColor *color, guchar alpha)
 {
 	g_return_if_fail (draw->started == TRUE);
-	g_return_if_fail (draw->impl != NULL);
 	g_return_if_fail (draw->impl->draw_text != NULL);
+
 	if (_vte_debug_on (VTE_DEBUG_DRAW)) {
 		GString *string = g_string_new ("");
 		gchar *str;
@@ -353,6 +346,7 @@ _vte_draw_text (struct _vte_draw *draw,
 				alpha);
 		g_free (str);
 	}
+
 	draw->impl->draw_text (draw, requests, n_requests, color, alpha);
 }
 
@@ -361,24 +355,31 @@ _vte_draw_char (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *request,
 	       GdkColor *color, guchar alpha)
 {
-	g_return_val_if_fail (draw->started == TRUE, FALSE);
-	g_return_val_if_fail (draw->impl != NULL, FALSE);
+	gboolean has_char;
+
 	_vte_debug_print (VTE_DEBUG_DRAW,
 			"draw_char ('%c', color=(%d,%d,%d,%d))\n",
 			request->c,
 			color->red, color->green, color->blue,
 			alpha);
-	if (draw->impl->draw_char == NULL) {
-		draw->impl->draw_text (draw, request, 1, color, alpha);
-		return TRUE;
-	}
-	return draw->impl->draw_char (draw, request, color, alpha);
+
+	has_char =_vte_draw_has_char (draw, request->c);
+	if (has_char)
+		_vte_draw_text (draw, request, 1, color, alpha);
+
+	return has_char;
 }
 gboolean
-_vte_draw_has_char (struct _vte_draw *draw, gunichar c)
+_vte_draw_has_char (struct _vte_draw *draw, vteunistr c)
 {
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_has_char ('%c')\n", c);
-	return draw->impl->has_char (draw, c);
+	gboolean has_char = TRUE;
+
+	_vte_debug_print (VTE_DEBUG_DRAW, "draw_has_char ('0x%04X')\n", c);
+
+	if (draw->impl->has_char)
+		has_char = draw->impl->has_char (draw, c);
+
+	return has_char;
 }
 
 void
@@ -387,13 +388,14 @@ _vte_draw_fill_rectangle (struct _vte_draw *draw,
 			 GdkColor *color, guchar alpha)
 {
 	g_return_if_fail (draw->started == TRUE);
-	g_return_if_fail (draw->impl != NULL);
 	g_return_if_fail (draw->impl->fill_rectangle != NULL);
+
 	_vte_debug_print (VTE_DEBUG_DRAW,
 			"draw_fill_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%d))\n",
 			x,y,width,height,
 			color->red, color->green, color->blue,
 			alpha);
+
 	draw->impl->fill_rectangle (draw, x, y, width, height, color, alpha);
 }
 
@@ -403,23 +405,23 @@ _vte_draw_draw_rectangle (struct _vte_draw *draw,
 			 GdkColor *color, guchar alpha)
 {
 	g_return_if_fail (draw->started == TRUE);
-	g_return_if_fail (draw->impl != NULL);
-	g_return_if_fail (draw->impl->draw_rectangle != NULL);
+
 	_vte_debug_print (VTE_DEBUG_DRAW,
 			"draw_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%d))\n",
 			x,y,width,height,
 			color->red, color->green, color->blue,
 			alpha);
-	draw->impl->draw_rectangle (draw, x, y, width, height, color, alpha);
-}
 
-void
-_vte_draw_set_scroll (struct _vte_draw *draw, gint x, gint y)
-{
-	g_return_if_fail (draw->impl != NULL);
-	g_return_if_fail (draw->impl->set_scroll != NULL);
-	_vte_debug_print (VTE_DEBUG_DRAW,
-			"draw_set_scroll (%d, %d)\n",
-			x, y);
-	draw->impl->set_scroll (draw, x, y);
+	if (draw->impl->draw_rectangle)
+		draw->impl->draw_rectangle (draw, x, y, width, height, color, alpha);
+	else {
+		if (width > 0) {
+			_vte_draw_fill_rectangle (draw, x, y, width-1, 1, color, alpha);
+			_vte_draw_fill_rectangle (draw, x+1, y+height-1, width-1, 1, color, alpha);
+		}
+		if (height > 0) {
+			_vte_draw_fill_rectangle (draw, x, y+1, 1, height-1, color, alpha);
+			_vte_draw_fill_rectangle (draw, x+width-1, y, 1, height-1, color, alpha);
+		}
+	}
 }
